# Doxicity 📗

Doxicity turns a directory of markdown files into HTML files. You can configure it, add data, layouts, partials, plugins, and use Handlebars `{{expressions}}` to do amazing things.

Designed in New Hampshire by [Cory LaViska](https://twitter.com/claviska).

---

## Usage

This package includes the Doxicity publishing engine. In the future, it may evolve to include a CLI and a dev server. For now, you can use it programmatically as shown below. Configuration options are shown with default values.

```ts
import Doxicity from 'doxicity';

//
// Configure it (all fields are optional)
//
const dox = new Doxicity({
  // The directory containing markdown files to publish.
  source: '.',

  // The directory where HTML files will be generated.
  destination: './_site',

  // The name of the directory that contains layouts. Must be relative to the project's source directory. This is where
  // *.hbs layouts are expected to be found. Defaults to "layouts".
  layouts: 'layouts',

  // The name of the directory that contains partials. Must be relative to the project's source directory. This is
  // where *.md and *.hbs partials are expected to be found. Defaults to "partials".
  partials: 'partials',

  // The name of the directory that contains the project's assets such as images, styles, scripts, etc. Must be relative
  // to the project's source directory. This directory will be copied to the output directory as-is on publish. Nothing
  // will be copied unless this option is set.
  assets: '',

  // Any data you put here will be available in your HTML and markdown files. If you use front matter in your markdown
  // files, that data will be merged with this data with the front matter taking precedence.
  data: {
    key: 'value',
    anotherKey: 'anotherValue'
  };

  // When true, Doxicity will delete the destination directory before writing anything to it.
  cleanBeforePublishing: false,

  // Optional plugins to customize how pages are published. Plugins are executed in the order they're provided in your
  // configuration.
  plugins: [],

  // Optional helpers to extend Handlebars with. To learn more about what helpers are and how they work, refer to the
  // Handlebars documentation. https://handlebarsjs.com/guide/expressions.html#helpers
  helpers: [],

  // A callback function that accepts a filename. If this function returns `true`, the target file will be published. If
  // it returns `false`, the file will be ignored.
  filter: () => true,

  // The extension to use when publishing pages. Defaults to `html`.
  extension: 'html';
});

//
// Publish it
//
dox
  .publish()
  .then(result => {
    console.log(
      `Pages published: ${result.pages.length}\n` +
        `Assets copied: ${result.assets.length}\n` +
        `Duration: ${result.timeToPublish}ms\n`
    );
  })
  .catch(err => {
    console.clear();
    console.log(err.message);
  });
```

## Pages

Pages are simply markdown files with optional front matter as shown below. You can use [Handlebars expressions](https://handlebarsjs.com/guide/expressions.html#expressions) to render data, loop through arrays, and more.

```md
---
title: My Page Title
---

# {{title}}

Markdown content goes here...
```

Published HTML pages will mirror the file structure of your markdown files.

## Layouts

Doxicity uses Handlebars as a templating engine. Layouts should be placed in the layouts folder, which is called `layouts` by default. Layout files should end in `.hbs` and can include HTML and [Handlebars expressions](https://handlebarsjs.com/guide/expressions.html#expressions). Avoid putting markdown in layout files, as it _will not_ be converted to HTML by Doxicity.

Layouts are used to scaffold pages. You can have as many layout files as you like, but the most common layout for your website should be called `default.hbs`. To indicate where content goes, use the `{{{$content}}}` expression as shown below. Note the triple brackets, which will render the content as HTML instead of escaped content.

```hbs
<html lang='en'>
  <head>
    <meta charset='UTF-8' />
    <meta http-equiv='X-UA-Compatible' content='IE=edge' />
    <meta name='viewport' content='width=device-width, initial-scale=1.0' />
    <title>{{title}}</title>
  </head>
  <body>
    <main>
      {{{$content}}}
    </main>
  </body>
</html>
```

Pages that include a `layout` field in their front matter will use the specified layout. The `.hbs` extension is optional in front matter. For example, the following page will use a layout located at `layouts/custom.hbs`.

```md
---
title: My Page Title
layout: custom
---

Markdown content goes here...
```

## Data

Two types of data can be provided to Doxicity. The first type is config data, which is globally available to all pages. The second type is page data, which is only available on the page it's provided on.

Config data is passed in through your Doxicity configuration. You can structure this data however you want, but you cannot prefix a field name with `$`, as those are reserved for magic properties.

Page data is defined in each page's front matter. Page data will be merged into config data when published, meaning page data can override config data if fields of the same name are used. This is by design, as it allows you to specify fallback values in your config and override them on selected pages.

## Partials

Partials are independent chunks of content that can be included in any template or page. They can be authored with a combination of markdown, HTML, and/or Handlebars expressions. Partials cannot have their own front matter, but they do have access to config data and page data. You can also pass data to them as hash arguments.

Partials are defined declaratively. To create a partial, create a file in the partials directory, which is called `partials` by default. Partial files can end with either `.md` or `.hbs`.

The name of the partial is its filename minus extension. To include a partial located at `partials/avatar.hbs` into a layout or page, use the following syntax.

```hbs
{{> avatar}}
```

You can group partials in subdirectories to keep things organized. If the partial is located at `partials/users/avatar.hbs`, include the path relative to the "partials" directory.

```hbs
{{> users/avatar}}
```

To pass data to a partial, use a hash argument. The following example passes a `name` and `image` argument to the partial.

```hbs
{{>avatar name="Doxicity" image="https://example.com/image.jpg"}}
```

## Assets

If you provide a value for `assets` in your config, Doxicity will copy the entire folder to the destination directory after publishing. The resulting directory will have the same name as the one being copied.

## Plugins

Plugins are used to extend Doxicity’s functionality. For your convenience, a number of useful [built-in plugins](https://github.com/claviska/doxicity/tree/next/src/plugins) come out of the box. You can enable as many or as few plugins as you’d like. Plugins are be executed in the same order they appear in the plugins config.

Example usage:

```ts
import Doxicity from 'doxicity';
import { activeLinks, anchorHeadings, codeHighlighter } from 'doxicity/dist/plugins.js';

const dox = new Doxicity({
  // ...
  plugins: [
    activeLinks({
      // optional config goes here
    }),
    anchorHeadings(),
    codeHighlighter()
  ]
});

dox.publish();
```

### Writing Plugins

Many static site generators offer a ton of hooks for plugin authors to tap into. However, this usually results in confusion and unintended results. For example, how does each hook affect rendering? What if a plugin runs too soon or too late? What if it only applies to markdown content and not the rest of the HTML? With so many hooks, it’s difficult to keep track of which plugins are doing what and when.

Doxicity simplifies this by offering a limited number of hooks and making it easy to mutate the entire page — not just the markdown content. At the heart of Doxicity’s plugin API is the `transform` hook. It provides you with a Document object that you can mutate just like you would in a browser.

Here’s a contrived example that prepends the text “Wow ” to every paragraph of every page.

```ts
import type { DoxicityPlugin } from 'doxicity';

const wowPlugin: DoxicityPlugin = {
  // Give it a name for debugging purposes
  name: 'doxicity-wow-plugin',

  // Use the transform hook to manipulate the DOM
  transform: doc => {
    doc.querySelectorAll('p').forEach(p => {
      p.innerHTML = 'Wow ' + p.innerHTML;
    });
  }
};

export default wowPlugin;
```

This example might seem silly, but the important thing to note is that you have complete control over every page. You can change the template’s structure, add or remove things from the DOM, mutate existing elements, and more. A good starting point for aspiring plugin authors is to look the [plugin folder](https://github.com/claviska/doxicity/tree/next/src/plugins) to see how some of the built-in plugins work.

### Plugin Hooks

The following hooks are currently available to plugin authors.

| Name              | Description                                                                                                                                                           |
| ----------------- | --------------------------------------------------------------------------------------------------------------------------------------------------------------------- |
| `transform`       | Hooks into the DOM transform phase, allowing you to mutate the document object before it gets turned into HTML. Use this for all types of DOM manipulation.           |
| `afterTransform ` | Hooks into the raw HTML string after all rendering and transformations are complete. Use this for formatters.                                                         |
| `afterAll`        | Hooks into the pages after they've been rendered and after all transforms have completed. Use this when you need to do something after all pages have been processed. |

Every Doxicity plugin must conform to the `DoxicityPlugin` interface. The interface is described in TypeScript, but you can write plugins in JavaScript as well. All hooks can be synchronous or asynchronous.

## Doxicity Flavored Markdown

Markdown is a lightweight markup syntax that turns text into HTML. It's a convenient way to go from things like `[this](this.html)` to `<a href="this.html">this<a>` without having to type all the quotes and brackets. It's built to be as simple as possible — most users can pick up the basics in just a few minutes.

Doxicity adheres to the [CommonMark spec](https://commonmark.org/), but there are a few additional syntaxes that are really helpful when writing docs.

### Markdown Cheat Sheet

The following table includes a non-exhaustive list of CommonMark syntaxes that are supported. You can expect these to work in Doxicity and most places markdown is supported.

<!-- prettier-ignore -->
| Feature         | Markdown                                                                                                                       | HTML            |
| --------------- | ------------------------------------------------------------------------------------------------------------------------------ | --------------- |
| Heading         | <pre>\# H1<br>\#\# H2<br>\#\#\# H3</pre>                                                                                       | `<h1>` – `<h6>` |
| Bold            | <pre>\*\*bold text\*\*</pre>                                                                                                   | `<strong>`      |
| Italic          | <pre>\*italicized text\*</pre>                                                                                                 | `<em>`          |
| Blockquote      | <pre>\> This is a blockquote</pre>                                                                                             | `<blockquote>`  |
| Ordered List    | <pre>1. Item 1<br>2. Item 2<br>3. Item 3</pre>                                                                                 | `<ol>` + `<li>` |
| Unordered List  | <pre>- Item 1<br>- Item 2<br>- Item 3</pre>                                                                                    | `<ul>` + `<li>` |
| Code (inline)   | <pre>\`inline code\`</pre>                                                                                                     | `<code>`        |
| Code (block)    | <pre>\`\`\`<br>code block<br>\`\`\`</pre>                                                                                      | `<pre>`         |
| Horizontal Rule | <pre>---</pre>                                                                                                                 | `<hr>`          |
| Link            | <pre>\[link text\](https://example.com/)</pre>                                                                                 | `<a>`           |
| Image           | <pre>\!\[alt text\](image.jpg)</pre>                                                                                           | `<img>`         |
| Table           | <pre>\| Syntax \| Description \|<br>\| ------ \| ----------- \|<br>\| Cell   \| Cell        \|<br>\| Cell   \| Cell        \|</pre> | `<table>`       |

### Doxicity-flavored Markdown

The following table includes additional Doxicity-specific markdown syntaxes. You should only expect these to work with Doxicity.

| Feature           | Markdown                                                                                          | HTML                                     |
| ----------------- | ------------------------------------------------------------------------------------------------- | ---------------------------------------- |
| Highlight         | <pre>\=\=mark\=\=</pre>                                                                           | `<mark>`                                 |
| Insert            | <pre>\+\+insert\+\+</pre>                                                                         | `<ins>`                                  |
| Delete/strike     | <pre>\~\~delete\~\~</pre>                                                                         | `<del>`                                  |
| Details           | <pre>:::details Toggle Me<br>This content is expandable<br>:::</pre>                              | `<details>` + `<summary>`                |
| Aside             | <pre>:::aside<br>This will be rendered on the side<br>:::</pre>                                   | `<aside>`                                |
| Tip callout\*     | <pre>:::tip<br>This is a tip!<br>:::</pre>                                                        | `<div class="callout callout--tip">`     |
| Warning callout\* | <pre>:::warning<br>This is a warning!<br>:::</pre>                                                | `<div class="callout callout--warning">` |
| Danger callout\*  | <pre>:::danger<br>This is something dangerous!<br>:::</pre>                                       | `<div class="callout callout--danger">`  |
| Cover\*           | <pre>:::cover<br>This is a cover section that will expand the full width of the page<br>:::</pre> | `<div class="cover">`                    |

Features marked with an asterisk apply container with classes. You will need to implement custom styles for best results.

## Previous Versions

Doxicity was recently rewritten to be more versatile and have less opinions. As such, this version is NOT compatible with previous releases tagged as 1.0.0-beta.
