import fs from 'fs/promises';
import path from 'path';
import cpy from 'cpy';
import merge from 'deepmerge';
import fm from 'front-matter';
import { globby } from 'globby';
import Handlebars from 'handlebars';
import { JSDOM } from 'jsdom';
import { AfterAllPluginError, AfterTransformPluginError, TransformPluginError, ReadError, WriteError } from './errors.js';
import { render as renderMarkdown } from './markdown.js';
import { render } from './render.js';
export default class Doxicity {
    constructor(config) {
        this.hasRegisteredPartials = false;
        this.config = {
            source: '.',
            destination: './_site',
            layouts: 'layouts',
            partials: 'partials',
            assets: '',
            data: {},
            filter: () => true,
            cleanBeforePublishing: false,
            extension: 'html',
            plugins: [],
            helpers: [],
            ...config
        };
        this.config.source = path.resolve(this.config.source);
        this.config.destination = path.resolve(this.config.destination);
        this.config.helpers.forEach(helper => {
            this.registerHelper(helper.name, helper.callback);
        });
    }
    async copyAssets() {
        if (this.config.assets) {
            const from = path.join(this.config.source, this.config.assets);
            const to = path.join(this.config.destination, this.config.assets);
            try {
                await fs.mkdir(to, { recursive: true });
                const copiedFiles = await cpy(from, to);
                return copiedFiles;
            }
            catch (err) {
                const message = `Doxicity wasn't able to copy assets from "${from}" to "${to}".\n\n` +
                    `Please verify that the target directories have the correct permissions and try again.`;
                throw new WriteError(message);
            }
        }
        return [];
    }
    getDestinationFilename(sourceFile) {
        const { dir, name } = path.parse(sourceFile);
        const outDir = path.join(this.config.destination, path.relative(this.config.source, dir));
        return path.join(outDir, `${name}.${this.config.extension}`);
    }
    getPathname(sourceFile) {
        const destinationFile = this.getDestinationFilename(sourceFile);
        const { pathname } = new URL(`https://internal/${path.relative(this.config.destination, destinationFile)}`);
        return pathname;
    }
    async parseSourceFile(sourceFile) {
        const data = await fs.readFile(sourceFile, 'utf-8');
        const { attributes, body } = fm(data);
        return {
            frontMatter: attributes,
            content: body
        };
    }
    async registerPartials() {
        const partialDir = path.resolve(this.config.source, this.config.partials);
        const partials = await globby([path.join(partialDir, '**/*.{hbs,md}')]);
        let partial = '';
        try {
            for await (partial of partials) {
                const parsed = path.parse(path.relative(partialDir, partial));
                const name = path.join(parsed.dir, parsed.name);
                const content = await fs.readFile(partial, 'utf-8');
                const html = renderMarkdown(content, { preserveHandlebars: true });
                Handlebars.registerPartial(name, html);
            }
        }
        catch (err) {
            const message = `Doxicity couldn't read the following partial from the file system.\n\n` +
                `${partial}\n\n` +
                `Please verify file permissions and try again.`;
            throw new ReadError(message);
        }
    }
    async renderPage(page) {
        return render(page, this.config);
    }
    registerHelper(name, callback) {
        Handlebars.registerHelper(name, callback);
    }
    registerPartial(name, htmlTemplate) {
        Handlebars.registerPartial(name, htmlTemplate);
    }
    async publish() {
        const startTime = Date.now();
        const ignoredDirectories = [
            path.join(this.config.source, this.config.layouts),
            path.join(this.config.source, this.config.partials)
        ];
        const files = (await globby([
            path.resolve(this.config.source, '**/*.md'),
            ...ignoredDirectories.map(dir => `!${path.resolve(this.config.source, dir)}/**/*.md`)
        ])).filter(file => this.config.filter(file));
        const publishedPages = [];
        if (!this.hasRegisteredPartials) {
            await this.registerPartials();
        }
        if (this.config.cleanBeforePublishing) {
            await fs.rm(this.config.destination, { recursive: true, force: true });
        }
        for await (const file of files) {
            const sourceFile = file;
            const destinationFile = this.getDestinationFilename(sourceFile);
            const pathname = this.getPathname(sourceFile);
            const { frontMatter, content } = await this.parseSourceFile(sourceFile);
            const data = merge(this.config.data, frontMatter);
            data.$content = content;
            data.$pathname = pathname;
            const page = {
                sourceFile,
                destinationFile,
                pathname,
                data
            };
            publishedPages.push(page);
            let html = await this.renderPage(page);
            const doc = new JSDOM(html, {
                url: `https://internal/`
            }).window.document;
            for (const plugin of this.config.plugins) {
                if (plugin.transform) {
                    try {
                        await plugin.transform(doc, page);
                    }
                    catch (err) {
                        const message = `A plugin called "${plugin.name} threw an error during the "transform" phase of the following page.\n\n` +
                            `${page.sourceFile}\n\n` +
                            `Here's some more info about the error.\n\n` +
                            `${err.message}`;
                        throw new TransformPluginError(plugin.name, page, message);
                    }
                }
            }
            html = doc.documentElement.outerHTML;
            for (const plugin of this.config.plugins) {
                if (plugin.afterTransform) {
                    try {
                        html = await plugin.afterTransform(html, page);
                    }
                    catch (err) {
                        const message = `A plugin called "${plugin.name} threw an error during the "afterTransform" phase of the following page.\n\n` +
                            `${page.sourceFile}\n\n` +
                            `Here's some more info about the error.\n\n` +
                            `${err.message}`;
                        throw new AfterTransformPluginError(plugin.name, page, message);
                    }
                    if (!html) {
                        const message = `A plugin called "${plugin.name} failed to return any HTML during the "afterTransform" phase of the following page.\n\n` +
                            `${page.sourceFile}\n\n` +
                            `Please update the plugin and try again.\n\n`;
                        throw new AfterTransformPluginError(plugin.name, page, message);
                    }
                }
            }
            try {
                await fs.mkdir(path.dirname(page.destinationFile), { recursive: true });
                await fs.writeFile(page.destinationFile, html, 'utf-8');
            }
            catch (err) {
                const message = `Doxicity couldn't write the following file to the file system.\n\n` +
                    `${page.destinationFile}\n\n` +
                    `Please verify that the target directory has the correct permissions and try again.`;
                throw new WriteError(message);
            }
        }
        const copiedAssets = await this.copyAssets();
        for await (const plugin of this.config.plugins) {
            if (plugin.afterAll) {
                try {
                    await plugin.afterAll(publishedPages);
                }
                catch (err) {
                    const message = `A plugin called "${plugin.name} threw an error during the "afterAll" phase.\n\n` +
                        `Here's some more info about the error.\n\n` +
                        `${err.message}`;
                    throw new AfterAllPluginError(plugin.name, message);
                }
            }
        }
        return {
            pages: publishedPages,
            assets: copiedAssets,
            timeToPublish: Date.now() - startTime
        };
    }
}
